@@ -1,7 +1,8 @@
 #!/bin/bash
-VERSION='0.3.3'
+VERSION='0.3.4'
 #
 # Copyright (C) 2003-2006  Ola Lundqvist <opal@debian.org>
+# Copyright (C) 2006       Miroslav Kure <kurem@upcase.inf.upol.cz>
 # Copyright (C) 2002       Mark Lawrence <nomad@null.net>
 # Copyright (C) 2002       Paul Sladen   <vserver@paul.sladen.org>
 #
@@ -17,8 +18,8 @@ VERSION='0.3.3'
 # 
 # You should have received a copy of the GNU General Public License with
 # the source package as the file COPYING.  If not, write to the Free
-# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-# 02111-1307, USA.
+# Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+# MA 02110-1301, USA.
 #
 # Install a virtual debian server (vserver) from a debian HTTP/FTP archive
 #
@@ -27,13 +28,21 @@ VERSION='0.3.3'
 
 shopt -s extglob
 
+# No context id set by default
+CONTEXT=
+
 # Root directory of your virtual servers (probably shouldn't change this)
-VROOTDIR="/vservers"
+# Support for 0.30.203 and above of util-vserver
+VROOTDIR="/etc/vservers/.defaults/vdirbase"
 
 # Packages to install in addition to the base defaults
 # MUST INCLUDE ALL DEPENDENCIES (seperated by "," commas)
 # Devfsd is very important.
-INSTALL_PACKAGES="less,dnsutils,wget,bzip2,ssh,rsync,libssl0.9.6,libdns5,libisc4,liblwres1,bind9-host,zlib1g,libbz2-1.0,traceroute,devfsd,nfsbooted"
+INSTALL_PACKAGES="nfsbooted"
+
+if [ -n "$LANG" ] && [ "$LANG" != "C" ] ; then
+    INSTALL_PACKAGES="$INSTALL_PACKAGES locales"
+fi
 
 # Packages installed from within the vserver after it has
 # been setup (seperated by "," commas)
@@ -49,8 +58,9 @@ REMOVE_PACKAGES="lilo,makedev,nfs-kernel-server,lpr"
 # otherwise dpkg will detect whether we are i386/powerpc/sparc/etc
 ARCH=""
 
-# Which debian distribution (warning: this has only been tested with woody and sarge)
-DIST="sarge"
+# Which debian distribution
+# (warning: this has only been tested with woody, sarge and etch)
+DIST="etch"
 
 # Local or nearest location of a debian mirror (must include the `/debian')
 MIRROR="http://ftp.uk.debian.org/debian"
@@ -58,9 +68,6 @@ MIRROR="http://ftp.uk.debian.org/debian"
 # Default network interface for vservers:
 INTERFACE="eth0"
 
-# Copy vreboot/vhalt/vshutdown utility into /usr/local/sbin/
-COPY_VREBOOT="true"
-
 if [ -r /etc/vservers/util-vserver-vars ] ; then
     . /etc/vservers/util-vserver-vars
 fi
@@ -99,9 +106,8 @@ Options:
         -V, --version		copyright and version information
 	--arch			set target architecture (eg. --arch "i386")
 				(autodetected on Debian host if dpkg available)
-	--copy-vreboot		install "vreboot/vshutdown/vhalt"
-	--no-copy-vreboot	don't install "vreboot/vshutdown/vhalt"
-	--dist			defaults to "sarge", passed to debootstrap.
+	--dist			defaults to "etch", passed to debootstrap.
+	--context n             Set the context id to be used.
 	--fakeinit		use "/sbin/init" to boot vserver
 	--conffile		extra configuration file to load.
 	--interface		interface for IP addresses (if not "eth0")
@@ -160,6 +166,10 @@ parse_args ()
 			esac
 			shift 2
 			;;
+		    --context)
+		        CONTEXT=$2
+			shift 2
+			;;
 		    --conffile)
 		        if [ -r "$2" ] ; then
 			    . "$2"
@@ -169,29 +179,21 @@ parse_args ()
 			fi
 			shift 2
 			;;
-		    --copy-vreboot)
-			COPY_VREBOOT="true"
-			shift
-			;;
 		    --nfs-server)
 		        NFS_SERVER=$2
 			shift 2
 			;;
-		    --no-copy-vreboot)
-			COPY_VREBOOT=""
-			shift
-			;;
 		    --dist)
 			case "$2" in
 			    [a-z]*)
 				DIST="$2"
-				if [ "sarge" != "$2" ] && [ "woody" != "$2" ] ; then
-				    echo "${0##*/} warning: I only know how to do \"woody\" and \"sarge\", be careful!" 1>&2
+				if [ "sarge" != "$2" ] && [ "woody" != "$2" ] && [ "etch" != "$2" ] ; then
+				    echo "${0##*/} warning: I only know how to do \"woody\", \"sarge\" and \"etch\", be careful!" 1>&2
 				fi
 				;;
 			    *)
 				echo "${0##*/} error: $1 requires a Debian distribution" 1>&2
-				echo 'e.g. "woody" or "sarge"' 1>&2
+				echo 'e.g. "woody", "sarge" or "etch"' 1>&2
 				exit 1
 				;;
 			esac
@@ -217,7 +219,7 @@ parse_args ()
 			;;
 		    --hostname)
 			case "$2" in
-			    *([a-z0-9])[a-z]*([a-z0-9]))
+			    [a-z0-9]*([a-z0-9_-])[a-z0-9])
 				VHOST="$2"
 				;;
 			    *)
@@ -288,13 +290,13 @@ parse_args ()
 				    VROOTDIR="$2"
 				else
 				    echo "${0##*/} error: $1 needs a valid absolute directory"  1>&2
-				    echo 'e.g. "/vservers"' 1>&2
+				    echo 'e.g. "/vservers or /etc/vservers/.defaults/vdirbase"' 1>&2
 				    exit 1
 				fi
 				;;
 			    *)
 				echo "${0##*/} error: $1 needs a valid absolute directory"  1>&2
-				echo 'e.g. "/vservers"' 1>&2
+				echo 'e.g. "/vservers or /etc/vservers/.defaults/vdirbase"' 1>&2
 				exit 1
 				;;
 			esac
@@ -368,26 +370,54 @@ if [ -n "$ARCH" ]; then
     ARCH_ARGUMENT="--arch $ARCH"
 fi
 
-# Fire off `debootstrap' to do all the hard work
-# like downloading and installing
-if ! /usr/sbin/debootstrap $ARCH_ARGUMENT \
-	"--include=$INSTALL_PACKAGES" "--exclude=$REMOVE_PACKAGES" \
-	"$DIST" "$VROOTDIR/$VHOST" "$MIRROR" ; then
-    echo "${0##*/}: error: debootstrap failure. Cannot continue."
+# Function to optionally set a context id.
+CONTEXT_ARGUMENT=
+if [ -n "$CONTEXT" ] ; then
+    CONTEXT_ARGUMENT="--context $CONTEXT"
+fi
+
+## use "vserver ... build" to build the new vserver
+if ! /usr/sbin/vserver "$VHOST" build -m debootstrap \
+       --rootdir "$VROOTDIR" --hostname "$VHOST" --interface "$INTERFACE:$IP" \
+       $CONTEXT_ARGUMENT \
+       -- -d "$DIST" -m "$MIRROR" \
+       -- $ARCH_ARGUMENT \
+       --include="$INSTALL_PACKAGES" --exclude="$REMOVE_PACKAGES"
+  then
+    echo "${0##*/}: error: vserver-build failure. Cannot continue."
     exit 1
 fi
 
+# Create a locale.gen if needed.
+if [ -n "$LANG" ] && [ "$LANG" != "C" ] ; then
+    if [ ! -e "$VROOTDIR/$VHOST/etc/locale.gen" ] ; then
+	echo $LANG $(locale charmap) > "$VROOTDIR/$VHOST/etc/locale.gen"
+    elif ! grep $LANG "$VROOTDIR/$VHOST/etc/locale.gen" > /dev/null ; then
+	echo $LANG $(locale charmap) >> "$VROOTDIR/$VHOST/etc/locale.gen"
+    fi
+fi
+
 # Make it so that apt and friends work
-cat << EOF > "$VROOTDIR/$VHOST/etc/apt/sources.list"
-deb $MIRROR/ $DIST main non-free contrib
-deb-src $MIRROR/ $DIST main non-free contrib
+if [ "woody" == "$DIST" ] ; then
+    cat << EOF > "$VROOTDIR/$VHOST/etc/apt/sources.list"
+deb $MIRROR/ $DIST main
+deb-src $MIRROR/ $DIST main
 
-deb http://non-us.debian.org/debian-non-US $DIST/non-US main contrib non-free
-deb-src http://non-us.debian.org/debian-non-US $DIST/non-US main contrib non-free
+deb http://non-us.debian.org/debian-non-US $DIST/non-US main
+deb-src http://non-us.debian.org/debian-non-US $DIST/non-US main
 
-deb http://security.debian.org $DIST/updates main contrib non-free
+deb http://security.debian.org $DIST/updates main
 
 EOF
+else
+       cat << EOF > "$VROOTDIR/$VHOST/etc/apt/sources.list"
+deb $MIRROR/ $DIST main
+deb-src $MIRROR/ $DIST main
+
+deb http://security.debian.org $DIST/updates main
+
+EOF
+fi
 
 # Fix up the available device nodes, for security
 if cd "$VROOTDIR/$VHOST/dev"; then
@@ -508,9 +538,17 @@ fi
 cat << EOF > $VROOTDIR/$VHOST/vserver-config.sh
 #!/bin/sh
 
+[ -x /usr/sbin/locale-gen ] && /usr/sbin/locale-gen
+
 dselect update
 
-tzsetup -y
+# tzsetup was part of base-config which is gone since etch
+# tzconfig is part of libc, so it should be ubiquitious
+if [ -x /usr/sbin/tzsetup ]; then
+    /usr/sbin/tzsetup -y
+elif [ -x /usr/sbin/tzconfig ]; then
+    /usr/sbin/tzconfig
+fi
 
 dpkg-reconfigure passwd
 
@@ -544,15 +582,6 @@ if [ -f "$SSH_KEYS" ]; then
     chmod 600 "$VROOTDIR/$VHOST/root/.ssh/authorized_keys"
 fi
 
-# Install the vreboot/rebootmgr utility--hopefully this will disappear soon
-VREBOOT_LOCATION="/usr/lib/util-vserver/vreboot"
-if [ "$COPY_VREBOOT" == "true" -a -x "$VREBOOT_LOCATION" ]; then
-    cp -a "$VREBOOT_LOCATION" "$VROOTDIR/$VHOST/usr/local/sbin/"
-    chmod 750 "$VROOTDIR/$VHOST/usr/local/sbin/vreboot"
-    # ln -s "$VROOTDIR/$VHOST/usr/local/sbin/"{vshutdown,vreboot}
-    # ln -s "$VROOTDIR/$VHOST/usr/local/sbin/"{vhalt,vreboot}
-fi
-
 # Stop all the processes that were started inside the server
 export PREVLEVEL=2
 vserver $VHOST exec /etc/init.d/rc 0
