@@ -1,6 +1,8 @@
 #!/bin/bash
+VERSION='0.3.4'
 #
 # Copyright (C) 2002-2006  Ola Lundqvist <opal@debian.org>
+# Copyright (C) 2006       Miroslav Kure <kurem@upcase.inf.upol.cz>
 # Copyright (C) 2005       Thomas Gelf <thomas@gelf.net>
 # Copyright (C) 2005       Daniel Hermann <hermann@tkm.physik.uni-karlsruhe.de>
 # Copyright (C) 2002       Paul Sladen   <vserver@paul.sladen.org>
@@ -18,8 +20,8 @@
 # 
 # You should have received a copy of the GNU General Public License with
 # the source package as the file COPYING.  If not, write to the Free
-# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-# 02111-1307, USA.
+# Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+# MA 02110-1301, USA.
 #
 # Install a virtual debian server (vserver) from a debian HTTP/FTP archive
 #
@@ -28,6 +30,9 @@
 
 shopt -s extglob
 
+# No context id set by default
+CONTEXT=
+
 # Root directory of your virtual servers (probably shouldn't change this)
 # Support for 0.30.203 and above of util-vserver
 VROOTDIR="/etc/vservers/.defaults/vdirbase"
@@ -36,6 +41,10 @@ VROOTDIR="/etc/vservers/.defaults/vdirbase"
 # MUST INCLUDE ALL DEPENDENCIES (seperated by "," commas)
 INSTALL_PACKAGES=""
 
+if [ -n "$LANG" ] && [ "$LANG" != "C" ] ; then
+    INSTALL_PACKAGES="$INSTALL_PACKAGES locales"
+fi
+
 # Packages installed from within the vserver after it has
 # been setup (seperated by "," commas)
 EXTRA_PACKAGES=""
@@ -55,8 +64,8 @@ POST_INSTALL_SCRIPT=""
 ARCH=""
 
 # Which debian distribution
-# (warning: this has only been tested with woody and sarge)
-DIST="sarge"
+# (warning: this has only been tested with woody, sarge and etch)
+DIST="etch"
 
 # Local or nearest location of a debian mirror (must include the `/debian')
 MIRROR="http://ftp.uk.debian.org/debian"
@@ -81,14 +90,14 @@ fi
 usage ()
 {
     cat << EOF 1>&2
-usage:	${0##*/} [OPTIONS] --hostname x --domain y.z --ip 1.2.3.4
+usage:	${0##*/} --hostname x --domain y.z --ip 1.2.3.4/24 [OPTIONS]
 	(see --help for more information)
 EOF
 }
 full_usage ()
 {
 	cat << EOF
-Usage:	${0##*/} [OPTIONS] --hostname x --domain y.z --ip 1.2.3.4/24
+Usage:	${0##*/} --hostname x --domain y.z --ip 1.2.3.4/24 [OPTIONS]
 Creates a new Debian vserver by calling "vserver ... build"
 
 Options:
@@ -96,7 +105,8 @@ Options:
         -V, --version		copyright and version information
 	--arch			set target architecture (eg. --arch "i386")
 				(autodetected on Debian host if dpkg available)
-	--dist			defaults to "sarge", passed to debootstrap.
+	--dist			defaults to "etch", passed to debootstrap.
+	--context n             Set the context id to be used.
 	--fakeinit		use "/sbin/init" to boot vserver
 	--conffile		extra configuration file to load.
 	--interface		interface for IP addresses (if not "eth0")
@@ -118,8 +128,8 @@ EOF
 full_version ()
 {
     cat << EOF
-${0##*/}
-Copyright (c) 2002-2005 Ola Lundqvist <opal@debian.org>
+${0##*/} version $VERSION
+Copyright (c) 2002-2006 Ola Lundqvist <opal@debian.org>
 Copyright (c) 2005      Daniel Hermann <hermann@tkm.physik.uni-karlsruhe.de>
 Copyright (c) 2002      Paul Sladen   <vserver@paul.sladen.org>
 Copyright (c) 2002      Mark Lawrence <nomad@null.net>
@@ -159,6 +169,10 @@ parse_args ()
 			esac
 			shift 2
 			;;
+		    --context)
+		        CONTEXT=$2
+			shift 2
+			;;
 		    --conffile)
 		        if [ -r "$2" ] ; then
 			    . "$2"
@@ -182,13 +196,13 @@ parse_args ()
 			case "$2" in
 			    [a-z]*)
 				DIST="$2"
-				if [ "sarge" != "$2" ] && [ "woody" != "$2" ] ; then
-				    echo "${0##*/} warning: I only know how to do \"woody\" and \"sarge\", be careful!" 1>&2
+				if [ "sarge" != "$2" ] && [ "woody" != "$2" ] && [ "etch" != "$2" ] ; then
+				    echo "${0##*/} warning: I only know how to do \"woody\", \"sarge\" and \"etch\", be careful!" 1>&2
 				fi
 				;;
 			    *)
 				echo "${0##*/} error: $1 requires a Debian distribution" 1>&2
-				echo 'e.g. "woody" or "sarge"' 1>&2
+				echo 'e.g. "woody", "sarge" or "etch"' 1>&2
 				exit 1
 				;;
 			esac
@@ -218,7 +232,7 @@ parse_args ()
 		        ;;
 		    --hostname)
 			case "$2" in
-			    *([a-z0-9])*([a-z0-9_-])[a-z]*([a-z0-9_-])*([a-z0-9]))
+			    [a-z0-9]*([a-z0-9_-])[a-z0-9])
 				VHOST="$2"
 				;;
 			    *)
@@ -383,29 +397,54 @@ if [ -n "$ARCH" ]; then
     ARCH_ARGUMENT="--arch $ARCH"
 fi
 
+# Function to optionally set a context id.
+CONTEXT_ARGUMENT=
+if [ -n "$CONTEXT" ] ; then
+    CONTEXT_ARGUMENT="--context $CONTEXT"
+fi
+
 ## use "vserver ... build" to build the new vserver
 if ! /usr/sbin/vserver "$VHOST" build -m debootstrap \
        --rootdir "$VROOTDIR" --hostname "$VHOST" --interface "$INTERFACE:$IP" \
+       $CONTEXT_ARGUMENT \
        -- -d "$DIST" -m "$MIRROR" \
        -- $ARCH_ARGUMENT \
-       --include="$INSTALL_PACKAGES" --exclude="$REMOVE_PACKAGES" \
-       --resolve-deps
+       --include="$INSTALL_PACKAGES" --exclude="$REMOVE_PACKAGES"
   then
     echo "${0##*/}: error: vserver-build failure. Cannot continue."
     exit 1
 fi
 
+# Create a locale.gen if needed.
+if [ -n "$LANG" ] && [ "$LANG" != "C" ] ; then
+    if [ ! -e "$VROOTDIR/$VHOST/etc/locale.gen" ] ; then
+	echo $LANG $(locale charmap) > "$VROOTDIR/$VHOST/etc/locale.gen"
+    elif ! grep $LANG "$VROOTDIR/$VHOST/etc/locale.gen" > /dev/null ; then
+	echo $LANG $(locale charmap) >> "$VROOTDIR/$VHOST/etc/locale.gen"
+    fi
+fi
+
 # Make it so that apt and friends work
-cat << EOF > "$VROOTDIR/$VHOST/etc/apt/sources.list"
-deb $MIRROR/ $DIST main non-free contrib
-deb-src $MIRROR/ $DIST main non-free contrib
+if [ "woody" == "$DIST" ] ; then
+    cat << EOF > "$VROOTDIR/$VHOST/etc/apt/sources.list"
+deb $MIRROR/ $DIST main
+deb-src $MIRROR/ $DIST main
 
-deb http://non-us.debian.org/debian-non-US $DIST/non-US main contrib non-free
-deb-src http://non-us.debian.org/debian-non-US $DIST/non-US main contrib non-free
+deb http://non-us.debian.org/debian-non-US $DIST/non-US main
+deb-src http://non-us.debian.org/debian-non-US $DIST/non-US main
 
-deb http://security.debian.org $DIST/updates main contrib non-free
+deb http://security.debian.org $DIST/updates main
 
 EOF
+else
+       cat << EOF > "$VROOTDIR/$VHOST/etc/apt/sources.list"
+deb $MIRROR/ $DIST main
+deb-src $MIRROR/ $DIST main
+
+deb http://security.debian.org $DIST/updates main
+
+EOF
+fi
 
 # Fix up the available device nodes (mostly done by vserver-build above)
 if cd "$VROOTDIR/$VHOST/dev"; then
@@ -524,9 +563,17 @@ fi
 cat << EOF > "$VROOTDIR/$VHOST/vserver-config.sh"
 #!/bin/sh
 
+[ -x /usr/sbin/locale-gen ] && /usr/sbin/locale-gen
+
 dselect update
 
-tzsetup -y
+# tzsetup was part of base-config which is gone since etch
+# tzconfig is part of libc, so it should be ubiquitious
+if [ -x /usr/sbin/tzsetup ]; then
+    /usr/sbin/tzsetup -y
+elif [ -x /usr/sbin/tzconfig ]; then
+    /usr/sbin/tzconfig
+fi
 
 dpkg-reconfigure passwd
 
