@@ -1,7 +1,8 @@
 #!/bin/bash
-VERSION='0.3.3'
+VERSION='0.3.4'
 #
 # Copyright (C) 2002-2006, Ola Lundqvist <opal@debian.org>
+# Copyright (C) 2006       Miroslav Kure <kurem@upcase.inf.upol.cz>
 # Copyright (C) 2002, Mark Lawrence <nomad@null.net>
 # Copyright (C) 2002, Paul Sladen   <vserver@paul.sladen.org>
 #
@@ -17,8 +18,8 @@ VERSION='0.3.3'
 # 
 # You should have received a copy of the GNU General Public License with
 # the source package as the file COPYING.  If not, write to the Free
-# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-# 02111-1307, USA.
+# Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+# MA 02110-1301, USA.
 #
 # Strip a server so it can function as a virtual debian server (vserver)
 #
@@ -28,11 +29,15 @@ VERSION='0.3.3'
 shopt -s extglob
 
 # Root directory of your virtual servers (probably shouldn't change this)
-VROOTDIR="/vservers"
+VROOTDIR="/etc/vservers/.defaults/vdirbase"
 
 # Packages to install in addition to the base defaults
 # MUST INCLUDE ALL DEPENDENCIES (seperated by "," commas)
-INSTALL_PACKAGES="less,dnsutils,wget,bzip2,ssh,rsync,libssl0.9.6,libdns5,libisc4,liblwres1,bind9-host,zlib1g,libbz2-1.0"
+INSTALL_PACKAGES=""
+
+if [ -n "$LANG" ] && [ "$LANG" != "C" ] ; then
+    INSTALL_PACKAGES="$INSTALL_PACKAGES locales"
+fi
 
 # Packages installed from within the vserver after it has
 # been setup (seperated by "," commas)
@@ -47,8 +52,9 @@ REMOVE_PACKAGES="dhcp-client,lilo,makedev,pcmcia-cs,ppp,pppconfig,pppoe,pppoecon
 # sysvinit services relating to hardware access to remove
 REMOVE_LINKS="klogd hwclock.sh setserial urandom networking umountfs halt reboot"
 
-# Which debian distribution (warning: this has only been tested with woody and sarge)
-DIST="sarge"
+# Which debian distribution
+# (warning: this has only been tested with woody, sarge and etch)
+DIST="etch"
 
 # Default network interface for vservers:
 INTERFACE="eth0"
@@ -96,7 +102,7 @@ Options:
 				(autodetected on Debian host if dpkg available)
 	--copy-vreboot		install "vreboot/vshutdown/vhalt"
 	--no-copy-vreboot	don't install "vreboot/vshutdown/vhalt"
-	--dist			defaults to "sarge", passed to debootstrap.
+	--dist			defaults to "etch", passed to debootstrap.
 	--fakeinit		use "/sbin/init" to boot vserver
 	--conffile		extra configuration file to load.
 	--interface		interface for IP addresses (if not "eth0")
@@ -162,13 +168,13 @@ parse_args ()
 			case "$2" in
 			    [a-z]*)
 				DIST="$2"
-				if [ "woody" != "$2" ] && [ "sarge" != "$2" ] ; then
-				    echo "${0##*/} warning: I only know how to do \"woody\" and \"sarge\", be careful!" 1>&2
+				if [ "woody" != "$2" ] && [ "sarge" != "$2" ] && [ "etch" != "$2" ] ; then
+				    echo "${0##*/} warning: I only know how to do \"woody\", \"sarge\" and \"etch\", be careful!" 1>&2
 				fi
 				;;
 			    *)
 				echo "${0##*/} error: $1 requires a Debian distribution" 1>&2
-				echo 'e.g. "woody" or "sarge"' 1>&2
+				echo 'e.g. "woody", "sarge" or "etch"' 1>&2
 				exit 1
 				;;
 			esac
@@ -194,7 +200,7 @@ parse_args ()
 			;;
 		    --hostname)
 			case "$2" in
-			    *([a-z0-9])[a-z]*([a-z0-9]))
+			    [a-z0-9]*([a-z0-9_-])[a-z0-9])
 				VHOST="$2"
 				;;
 			    *)
@@ -252,13 +258,13 @@ parse_args ()
 				    VROOTDIR="$2"
 				else
 				    echo "${0##*/} error: $1 needs a valid absolute directory"  1>&2
-				    echo 'e.g. "/vservers"' 1>&2
+				    echo 'e.g. "/vservers or /etc/vservers/.defaults/vdirbase"' 1>&2
 				    exit 1
 				fi
 				;;
 			    *)
 				echo "${0##*/} error: $1 needs a valid absolute directory"  1>&2
-				echo 'e.g. "/vservers"' 1>&2
+				echo 'e.g. "/vservers or /etc/vservers/.defaults/vdirbase"' 1>&2
 				exit 1
 				;;
 			esac
@@ -328,6 +334,15 @@ if cd "$VROOTDIR/$VHOST/dev"; then
     mkdir shm
 fi
 
+# Create a locale.gen if needed.
+if [ -n "$LANG" ] && [ "$LANG" != "C" ] ; then
+    if [ ! -e "$VROOTDIR/$VHOST/etc/locale.gen" ] ; then
+	echo $LANG $(locale charmap) > "$VROOTDIR/$VHOST/etc/locale.gen"
+    elif ! grep $LANG "$VROOTDIR/$VHOST/etc/locale.gen" > /dev/null ; then
+	echo $LANG $(locale charmap) >> "$VROOTDIR/$VHOST/etc/locale.gen"
+    fi
+fi
+
 # Give the new host a hostname
 if ! grep $VHOST "$VROOTDIR/$VHOST/etc/hostname" > /dev/null ; then
     echo "Hostname not found in hostname file, rewriting and creating backup."
@@ -486,6 +501,8 @@ fi
 cat << EOF > "$VROOTDIR/$VHOST/vserver-config.sh"
 #!/bin/sh
 
+[ -x /usr/sbin/locale-gen ] && /usr/sbin/locale-gen
+
 dselect update
 
 ###tzsetup -y
